#!/usr/bin/env python3
import numpy as np
import numpy.linalg as alg
from tests import test_descente, run_test


def descente(A, B):
    """
    for some reason I can't get all 1000 tests to pass
    getting some numerical instability and stuff
    """
    if alg.det(A) == 0:
        return None
    m = np.shape(B)[0]
    m_idx = m - 1
    X = np.transpose(np.matrix(np.zeros(m)))
    X[0] = B[0] / A[0, 0]
    for i in range(1, m):
        if A[m_idx, m_idx] == 0 or A[i, i] == 0:
            continue
        sum = 0
        for j in range(i):
            sum += A[i, j] * X[j]
        X[i] = (B[i] - sum) / A[i, i]
    return X


def main():
    run_test(test_descente, 5_000, descente)


if __name__ == '__main__':
    main()



import numpy as np
import matplotlib.pyplot as plt

def f(x):
    return x**3 - x -1

def fprime(x):
    return 3*x**2 - 1

x = 0

epsilon = 0.000001
nb_iterations = 0
x0 = 0
x = x0

while abs(f(x)) > epsilon:
    x = x - f(x) / fprime(x)

    nb_iterations += 1

print(round(x, 4))
print(nb_iterations)
print(x0)



#!/usr/bin/env python3

import numpy as np
from remontee import remontee
from tests import test_gauss, run_test


def gauss(A_in, B_in):
    """
    can't pass all tests every time...
    """
    # we make a copy to avoid modifying the original matrices
    A, B = A_in.copy(), B_in.copy()
    m = np.shape(B)[0]
    g = np.matrix(np.zeros((m, m)))
    for k in range(m):
        if A[k, k] == 0:
            continue
        for i in range(k + 1, m):
            g[i, k] = A[i, k] / A[k, k]
            for j in range(k, m):
                # you can start at k + 1 but starting at k
                # adds zeros underneath the diagonal, showing
                # the matrix is indeed upper triangular
                A[i, j] = A[i, j] - g[i, k] * A[k, j]
            B[i] = B[i] - g[i, k] * B[k]
    return A, B


"""
This code FAILS for the following matrices A & B:


sage: A=matrix([[1, 2, 0, 0, 0],
....:               [2, -9, 3, 4, 0],
....:               [0, 3, -12, 4, 5],
....:               [0, 4, 4, -9, 5],
....:               [0, 0, 5, 5, -5]])
sage: A
[  1   2   0   0   0]
[  2  -9   3   4   0]
[  0   3 -12   4   5]
[  0   4   4  -9   5]
[  0   0   5   5  -5]
sage: B=column_matrix([1,4,9,16,25])
sage:
sage: B
[ 1]
[ 4]
[ 9]
[16]
[25]
sage: A.solve_right(B)
[-6509/1053]
[ 3781/1053]
[  2203/351]
[ 7858/1053]
[ 9202/1053]

"""


def main():
    run_test(test_gauss, 5_000, gauss, remontee)


if __name__ == '__main__':
    main()



import numpy as np
import numpy.linalg as alg
from random import randint
from enum import Enum


MatrixType = Enum('MatrixType', [
    'RANDOM',
    'LOWER_TRIANGULAR',
    'UPPER_TRIANGULAR',
    'SYMMETRIC_POSITIVE_DEFINITE'
])


def generate_matrix(matrix_type: MatrixType, n=-1):
    if n == -1:
        n = randint(2, 12)  # default random size
    A = np.matrix(np.zeros((n, n)))
    B = np.transpose(np.matrix(np.zeros(n)))
    must_retry = False
    while alg.det(A) == 0 or must_retry:
        if matrix_type == MatrixType.SYMMETRIC_POSITIVE_DEFINITE:
            for i in range(n):
                for j in range(i, n):
                    A[i, j] = randint(0, 99) / 10
                    A[j, i] = A[i, j]
                B[i] = randint(-99, 99) / 10
            try:
                alg.cholesky(A)
                must_retry = False
            except alg.LinAlgError:
                must_retry = True
                continue
        else:
            for i in range(n):
                range_j = range(n)
                if matrix_type == MatrixType.LOWER_TRIANGULAR:
                    range_j = range(i + 1)
                elif matrix_type == MatrixType.UPPER_TRIANGULAR:
                    range_j = range(i, n)
                for j in range_j:
                    A[i, j] = randint(-99, 99) / 10
                B[i] = randint(-99, 99) / 10
    return A, B


def display_fail(A, B, got, expected):
    print("FAILED TEST")
    print("A:")
    print(A)
    print("B:")
    print(B)
    print("got: \n", got)
    print("expected:\n", expected)


def test_remontee(remontee, show_failed=False):
    A, B = generate_matrix(MatrixType.UPPER_TRIANGULAR)
    result = remontee(A, B)
    expected = alg.solve(A, B)
    passed = np.allclose(result, expected)
    if not passed and show_failed:
        display_fail(A, B, result, expected)
    return passed


def test_descente(descente, show_failed=False):
    A, B = generate_matrix(MatrixType.LOWER_TRIANGULAR)
    result = descente(A, B)
    expected = alg.solve(A, B)
    passed = np.all(np.abs(result - expected) < 10**-6)
    if not passed and show_failed:
        display_fail(A, B, result, expected)
    return passed


def test_gauss(gaussian_elim, remontee, show_failed=False):
    A, B = generate_matrix(MatrixType.RANDOM)
    trig_A, trig_B = gaussian_elim(A, B)
    result = remontee(trig_A, trig_B)
    expected = alg.solve(A, B)
    passed = False
    try:
        passed = np.all(np.abs(result - expected) < 10**-6)
    except Exception as e:
        print(e)
    if not passed and show_failed:
        display_fail(A, B, result, expected)
    return passed


def test_cholesky(cholesky, cholesky_solve, show_failed=False):
    A, B = generate_matrix(MatrixType.SYMMETRIC_POSITIVE_DEFINITE, randint(2, 5))
    cholesky_decomp = cholesky(A)
    expected_decomp = alg.cholesky(A)
    passed = np.allclose(cholesky_decomp, expected_decomp)
    if not passed and show_failed:
        print("CHOLESKY DECOMPOSITION FAIL")
        display_fail(A, B, cholesky_decomp, expected_decomp)
    result_solve = cholesky_solve(cholesky_decomp, B)
    expected_solve = alg.solve(A, B)
    passed_solve = np.allclose(result_solve, expected_solve)
    if not passed_solve and show_failed:
        print("CHOLESKY SOLVE FAIL")
        display_fail(A, B, result_solve, expected_solve)
    return passed and passed_solve


def run_test(test_function, num_tests, *args):
    tests_passed = 0
    for i in range(num_tests):
        if i % (num_tests // 5) == 0 and i != 0:
            print(f"Running test {i}...")
            success_rate = tests_passed / i * 100
            print(f"> Passed so far: {tests_passed}/{i} (success rate: {success_rate:.2f}%)\n")
        tests_passed += test_function(*args)
    success_rate_tot = tests_passed / num_tests * 100
    print(f"\n\n\nTests passed: {tests_passed}/{num_tests} (success rate: {success_rate_tot:.2f}%)")



import numpy as np
import matplotlib.pyplot as plt

x = np.linspace(0,2,1000)

def f(x):
    return x**3 - x -1

a, b = 1, 2
epsilon = 0.0001

x_mid = 0

while b - a >= epsilon:
    x_mid = (b + a)/2

    if f(a)*f(x_mid) < 0:
        b = x_mid
    else:
        a = x_mid

print("Solution = ", x_mid)


import numpy as np

A = np.array([[-6, 1, -1, 0],
              [1, 4, 0, 1],
              [-1, 0, -6, 2],
              [0, 1, 2, 6]])
              
n = len(A)

B = np.array([-3, 9, -2, 0])

X = np.zeros(n)

D = np.zeros((n, n))
L = np.zeros((n, n))
U = np.zeros((n, n))

for i in range(len(A)):
    for j in range(len(A)):
        if i == j:
            D[i][j] = A[i][j]
        elif i > j:
            L[i][j] = -A[i][j]
        elif j > i:
            U[i][j] = -A[i][j]

epsilon = 0.001
omega = 0.5
r = B - np.dot(A, X)
ca = np.linalg.norm(r) / np.linalg.norm(B)

X_new = X

while ca >= epsilon:
    X_new = np.dot(np.linalg.inv(D-L), np.dot(U,X) + B)
    X = X + omega * (X_new - X)
    
    r = B - np.dot(A, X)
    ca = np.linalg.norm(r) / np.linalg.norm(B)

print(X)


import numpy as np
import math as m

station1 = np.array([2.8, 10.2, 5.2])
station2 = np.array([14, 13, 7])
station3 = np.array([4, 15, 3.5])

A = np.array([[-2*station1[0], -2*station1[1], 1],
              [-2*station2[0], -2*station2[1], 1],
              [-2*station3[0], -2*station3[1], 1]])

B = np.array([station1[2] - m.pow(station1[0], 2) - m.pow(station1[1], 2),
              station2[2] - m.pow(station2[0], 2) - m.pow(station2[1], 2),
              station3[2] - m.pow(station3[0], 2) - m.pow(station3[1], 2)])

n = len(A)
X = np.zeros(n)

for k in range(n):

    if A[k][k] == 0:
        for i in range(k+1, n):
            if A[i][k] != 0:
                A[k], A[i] = A[i], A[k]
                B[k], B[i] = B[i], B[k]
                break
                
    for i in range(k+1, n):
        g = A[i][k] / A[k][k]
        for j in range(k+1, n):
            A[i][j] = A[i][j] - g * A[k][j]
        B[i] = B[i] - g * B[k]
        A[i][k] = 0

X[n-1] = B[n-1] / A[n-1][n-1]

for i in range(n-2, -1, -1):
    somme = 0
    for j in range(i+1, n):
        somme += A[i][j] * X[j]
    X[i] = (B[i] - somme) / A[i][i]

print("Position telephone : \n", X)



#!/usr/bin/env python3

import numpy as np
from remontee import remontee
from descente import descente
from tests import test_cholesky, run_test


def cholesky(A):
    m = np.shape(A)[0]
    L = np.matrix(np.zeros((m, m)))
    #
    # first column
    #
    L[0, 0] = np.sqrt(A[0, 0])
    for i in range(1, m):
        L[i, 0] = A[i, 0] / L[0, 0]
    for k in range(1, m):
        #
        # l_kk coefficients (p.14)
        #
        sum_0 = 0
        for j in range(k):
            sum_0 += np.pow(L[k, j], 2)
        L[k, k] = np.sqrt(A[k, k] - sum_0)
        #
        # l_ik coefficients
        #
        for i in range(k, m):
            sum_1 = 0
            for j in range(k):
                sum_1 += L[i, j] * L[k, j]
            L[i, k] = (A[i, k] - sum_1) / L[k, k]
    return L


def cholesky_solve(L, B):
    # two systems to solve:
    # LY = B
    # L^T X = Y
    Y = descente(L, B)
    X = remontee(np.transpose(L), Y)
    return X


def main():
    run_test(test_cholesky, 200, cholesky, cholesky_solve)


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

setup(
    name="ResolutionSystemesLineaireEtNon",
    version="0.1.1",
    packages=find_packages(),
    install_requires=["numpy", "matplotlib"],  # Dépendances à installer
    description="Description de mon package",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Leo leo",
    author_email="hocomi3722@exeneli.com",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



#!/usr/bin/env python3
import numpy as np
import numpy.linalg as alg
from tests import test_remontee, run_test


def remontee(A, B):
    """
    passes all tests every time
    """
    if alg.det(A) == 0:
        return None
    m = np.shape(B)[0]
    m_idx = m - 1
    X = np.transpose(np.matrix(np.zeros(m)))
    X[m_idx] = B[m_idx] / A[m_idx, m_idx]
    for i in range(m_idx - 1, -1, -1):  # iterate from m_idx to 0 inclusive
        if A[m_idx, m_idx] == 0 or A[i, i] == 0:
            continue
        sum = 0
        for j in range(i + 1, m_idx + 1):   # sums are inclusive in math!!!
            # watch out for that + 1
            sum += A[i, j] * X[j]
        X[i] = (B[i] - sum) / A[i, i]
    return X


def main():
    run_test(test_remontee, 5000, remontee)


if __name__ == '__main__':
    main()
